/*
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

/*
 * EE to test channels
 */

#undef CHAN_ETH
#define USEIFCHAN	/* use "if" instead of "eth" */

#include <stdio.h>
#include <string.h>
#include <assert.h>

#include <nodeos/impl.h>
#include <nodeos/flow.h>
#include <nodeos/thread.h>
#include <nodeos/channel.h>
#include <nodeos/misc.h>

static void ee_init(void *arg);
an_flowinit_t InitEE = { ee_init };

static void setup_channels(void);
static void ee_packet1(an_packet_t packet, void *arg);
static void ee_packet100(an_packet_t packet, void *arg);
static an_chan_t chan1, chan100;
#ifdef CHAN_ETH
static void ee_eth(void);
static void ee_realpacket(an_packet_t packet, void *arg);
static an_ifinfo_t info[8];
#endif

static struct {
	an_chanaddr_t addr;
	an_chanproto_t proto;
	an_cpuspec_t cpu;
	an_memspec_t mem;
} attr;
static an_flow_t flow;

int
ee_main(int argc, char **argv)
{
	flow = an_flow_current();
	printf("%p/%p: Channel test EE started\n",
	       flow, an_thread_current());

	/*
	 * Setup channels
	 */
	setup_channels();

	/*
	 * No longer need this thread
	 */
	return 0;
}

static void
setup_channels(void)
{
	memset(&attr, 0, sizeof attr);

#ifdef CHAN_ETH
	ee_eth();
#endif

	strcpy(attr.proto.data, "clock");
	attr.proto.len = strlen(attr.proto.data);
	attr.addr.clock.freq = 100;
	attr.cpu.max = 1;
	attr.mem.npages = 1;
	chan100 = an_inchan_create(flow, 0, &attr.proto, &attr.addr,
				   &attr.mem, &attr.cpu, ee_packet100, &chan100);
	assert(chan100 != 0);

	attr.addr.clock.freq = 1;
	chan1 = an_inchan_create(flow, 0, &attr.proto, &attr.addr,
				 &attr.mem, &attr.cpu, ee_packet1, &chan1);
	assert(chan1 != 0);
}

static int inpackets;

static void
ee_packet1(an_packet_t packet, void *arg)
{
	assert(arg == &chan1);

	inpackets++;
	an_packet_return(packet);
}

static void
ee_packet100(an_packet_t packet, void *arg)
{
	static int count;
	int size = an_packet_size(packet);
	unsigned int *ts = an_packet_data(packet);
	an_timespec_t tspec;
	
	assert(arg == &chan100);
	assert(size == sizeof *ts);

	an_gettime(&tspec);
	printf("%p/%p: packet %p@%u: got %d packets at %u seconds\n",
	       an_flow_current(), an_thread_current(), packet, *ts, inpackets,
	       (unsigned int)tspec.tv_sec);
	an_packet_return(packet);
	if (++count == 20) {
		printf("killing timer channel %p\n", chan1);
		if (chan1)
			an_chan_destroy(chan1);
/* if we kill our channel, the flow won't terminate */
#if 0
		printf("killing our channel %p\n", chan100);
		if (chan100)
			an_chan_destroy(chan100);
#else
		an_flow_destroy(an_flow_current());
#endif
		printf("WE ARE DEAD!\n");
	}
}

#ifdef CHAN_ETH
static void
ee_eth(void)
{
	int nif = 8;
	int err, i;
	an_chan_t eth;

	/*
	 * Find out interfaces.
	 */
	err = an_getifinfo(info, &nif);
	assert(err == 0);

	for (i = 0; i < nif; i++) {
		assert(info[i].type == ANIF_ETH);

#if 1
		printf("if%d: %s: %02x:%02x:%02x:%02x:%02x:%02x\n",
		       info[i].id, info[i].pname,
		       info[i].addr.eth.daddr[0], info[i].addr.eth.daddr[1],
		       info[i].addr.eth.daddr[2], info[i].addr.eth.daddr[3],
		       info[i].addr.eth.daddr[4], info[i].addr.eth.daddr[5]);
#endif
#ifdef USEIFCHAN
		sprintf(attr.proto.data, "if%d", info[i].id);
#if 1
		/* get broadcasts as well as directed packets */
		memset(&attr.addr, 0, sizeof attr.addr);
#else
		/* directed packets only */
		attr.addr = info[i].addr;
#endif
#else
		strncpy(attr.proto.data, info[i].pname, sizeof info[i].pname);
#endif
		attr.proto.len = strlen(attr.proto.data);
		attr.cpu.max = 2;
		attr.mem.npages = 1;
		eth = an_inchan_create(flow, 0, &attr.proto, &attr.addr,
				       &attr.mem, &attr.cpu, ee_realpacket,
				       &info[i]);
		assert(eth != 0);
	}
}

static void
ee_realpacket(an_packet_t packet, void *arg)
{
	an_ifinfo_t *info = arg;
#ifdef USEIFCHAN
	unsigned int *ip = an_packet_data(packet);

	assert(info != 0);
	printf("%p/%p: if%d: %8.8x %8.8x\n",
	       an_flow_current(), an_thread_current(), info->id, ip[0], ip[1]);
#else
	int size = an_packet_size(packet);
	struct {
		unsigned char daddr[6];
		unsigned char saddr[6];
		short type;
	} *hdr = an_packet_data(packet);

	assert(info != 0);
	assert(size >= sizeof *hdr);

	printf("%p/%p: %s: "
	       "%02x:%02x:%02x:%02x:%02x:%02x -> %02x:%02x:%02x:%02x:%02x:%02x"
	       ": %x\n",
	       an_flow_current(), an_thread_current(), info->pname,
	       hdr->saddr[0], hdr->saddr[1], hdr->saddr[2],
	       hdr->saddr[3], hdr->saddr[4], hdr->saddr[5],
	       hdr->daddr[0], hdr->daddr[1], hdr->daddr[2],
	       hdr->daddr[3], hdr->daddr[4], hdr->daddr[5], ntohs(hdr->type));
#endif

	an_packet_return(packet);
}
#endif

static void
ee_init(void *arg)
{
	int status;
	int argc = 1;
	char *argv[] = { "ChantestEE" }; 

	status = ee_main(argc, argv);
}

