#
# Copyright (c) 1999, 2000 University of Utah and the Flux Group.
# All rights reserved.
# This file is part of Janos.
# 
# Janos is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License, version 2, as
# published by the Free Software Foundation.
# 
# Janos is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details, in the file
# COPYING.  If that file is not present, write to the Free
# Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
# 02111-1307, USA
# 
# The University of Utah and the Flux Group requests users to
# return to flux-dist@cs.utah.edu any improvements that they make.
#

#
# Assumes EE and EESRCS are defined by "caller".
#
# Also expects SRCDIR, LD, LDFLAGS, CC, CFLAGS

$(EE)_EXE :=	$(EE)
$(EE)_OEXE :=	$(EE)_o
$(EE)_PEXE :=	$(EE)_p

$(EE)_SRCS := 		$(EESRCS)
$(EE)_DEPSRCS :=	$(addprefix $(SRCDIR)/,$($(EE)_SRCS))

$(EE)_OBJS :=	$(patsubst %.c,%.o,$(notdir $($(EE)_SRCS)))
$(EE)_OOBJS :=	$(patsubst %.c,%.oo,$(notdir $($(EE)_SRCS)))
$(EE)_POBJS :=	$(patsubst %.c,%.po,$(notdir $($(EE)_SRCS)))

$($(EE)_EXE)_LDFLAGS :=	$(EELDFLAGS)
$($(EE)_OEXE)_LDFLAGS :=	$(EELDFLAGS)
$($(EE)_PEXE)_LDFLAGS :=	$(EELDFLAGS)

## Build the vanilla executable
$($(EE)_EXE): $($(EE)_OBJS) $(NODEOS_LIB_DEPS)
	$(LD) $(LDFLAGS) -o $@ \
		$(filter-out %.a,$^) \
		$($(@)_LDFLAGS) $(NODEOS_LIBS) $(ARCH_LIBS)

## Bulid the optimized executable
$($(EE)_OEXE): $($(EE)_OOBJS) $(NODEOS_LIB_DEPS)
	$(LD) $(LDFLAGS) -o $@ \
		$(filter-out %.a,$^) \
		$($(@)_LDFLAGS) $(NODEOS_LIBS) $(ARCH_LIBS)

## Build the profiling executable
$($(EE)_PEXE): $($(EE)_POBJS) $(NODEOS_LIB_DEPS)
	$(LD) $(LDFLAGS) -o $@ \
		$(filter-out %.a,$^) \
		$($(@)_LDFLAGS) $(NODEOS_LIBS) $(ARCH_PLIBS)

$(EE)_mbimg: $($(EE)_EXE)
	mkmbimage -o $@ $^

$(EE)_mbimg_o: $($(EE)_OEXE)
	mkmbimage -o $@ $^

$(EE)_mbimg_p: $($(EE)_PEXE)
	mkmbimage -o $@ $^

$(EE)_armimg: $($(EE)_EXE)
	@cp -f $^ $(^)_stripped
	$(STRIP) --strip-debug $(^)_stripped
	$(OSKIT_INSTDIR)/bin/mkofwimage -o $@ $(^)_stripped


PHONY: $(EE).debugmake gendepend.$(EE)

depend:: gendepend.$(EE)

gendepend.$(EE): $($(EE)_DEPSRCS)
	@echo "Generating dependencies for $(subst gendepend.,,$@)"
	@$(CC) $(CFLAGS) -E -M $(CPPFLAGS) $^ $(DEPINCS) > $(subst gendepend.,.depend.,$@)

-include .depend.$(EE)


%.debugmake:
	@echo define_ee.mk debugging for EE \"$*\"
	@echo "  sources = $($(*)_SRCS)"
	@echo "  target  = $($(*)_EXE)"
	@echo "  extra LD flags= $($(*)_LDFLAGS)"
	@echo "  debugging = $($(*)_EXE) <- $($(*)_OBJS)"
	@echo "  optimized = $($(*)_OEXE) <- $($(*)_OOBJS)"
	@echo "  profiling = $($(*)_PEXE) <- $($(*)_POBJS)"
	@echo "  depend file = .depend.$(*)"
