/*
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

/*
 * Open a receive channel, wait til they type "exit"
 */
#undef BLOW
#define BLOWHOST	0x9b63d658	/* moab.cs.utah.edu */

#include <stdio.h>
#include <assert.h>
#include <string.h>
#include <sys/types.h>

#include <sys/socket.h>
#include <netinet/in.h>

#include <nodeos/impl.h>
#include <nodeos/flow.h>
#include <nodeos/thread.h>
#include <nodeos/channel.h>

#define RUNFOR		30	/* run for this many seconds */

#define TTCPPORT	5001	/* default ttcp port */
static unsigned char myip[4] = { 155, 99, 214, 158 };

static void ee_init(void *arg);
an_flowinit_t InitEE = { ee_init };

static void ee_die(an_packet_t packet, void *arg);
static void setup_channels(void);

static an_chan_t die;
#ifdef BLOW
static an_chan_t udpoutchan;
#else
static void ee_udp(an_packet_t packet, void *arg);
static void ee_ttcp(an_packet_t packet, void *arg);
static an_chan_t udpinchan, ttcpinchan;
static struct {
	int off;
	int len;
	char data[8];
} ttcppattern = { 0, 8, { 040, 041, 042, 043, 044, 045, 046, 047 } };
static an_demuxkey_t ttcpdemux;
#endif
static unsigned int myipaddr;
static int npackets, ttcppackets, chandead;
static unsigned long nbytes;

int
ee_main(int argc, char **argv)
{
	assert(argc >= 1);

	printf("%p/%p: %s started\n",
	       an_flow_current(), an_thread_current(), argv[0]);
	myipaddr = htonl(*(unsigned int *)myip);

	/*
	 * Setup channels
	 */
	setup_channels();

	printf("ready\n");

#ifdef BLOW
	{
		static void ee_sendudp(void);
		ee_sendudp();
	}
#endif

#ifdef GPROF
	oskit_start_gprof();
#endif
#ifdef OSKIT
	{
		extern int memlogging;
		memlogging = 1;
	}
#endif
	return 0;
}

static void
setup_channels(void)
{
	struct {
		an_chanaddr_t addr;
		an_chanproto_t proto;
		an_cpuspec_t cpu;
		an_memspec_t mem;
	} attr;
	struct sockaddr_in *addr;
	an_error_t err;
	an_flow_t flow = an_flow_current();

	memset(&attr, 0, sizeof attr);

	/*
	 * Setup a clock channel to time us out in RUNFOR seconds
	 */
	strcpy(attr.proto.data, "clock");
	attr.proto.len = strlen(attr.proto.data);
	attr.addr.clock.freq = (RUNFOR * 100);
	attr.cpu.max = 2;
	attr.mem.npages = 1;
	die = an_inchan_create(flow, 0, &attr.proto, &attr.addr,
			       &attr.mem, &attr.cpu, ee_die, 0);
	assert(die != 0);
	printf("%p: clockchannel for %d seconds\n", die, RUNFOR);
	
#ifdef BLOW
	/*
	 * Setup out channel for UDP
	 */
	strcpy(attr.proto.data, "socket/udp/ip/eth");
	attr.proto.len = strlen(attr.proto.data);
	addr = (struct sockaddr_in *)attr.addr.data;
	addr->sin_addr.s_addr = htonl(BLOWHOST);
	addr->sin_family = AF_INET;
	addr->sin_port = htons(TTCPPORT);
	attr.addr.len = sizeof *addr;
	udpoutchan = an_outchan_create(flow, &attr.proto, &attr.addr, 0);
	assert(udpoutchan != 0);
	printf("%p: outchannel for port=%d\n", udpoutchan, TTCPPORT);
#else
	/*
	 * Setup in channels for UDP and UDP-based TTCP
	 */
	strcpy(attr.proto.data, "socket/udp/ip/eth");
	attr.proto.len = strlen(attr.proto.data);
	addr = (struct sockaddr_in *)&attr.addr.socket;
	addr->sin_addr.s_addr = htonl(INADDR_ANY);
	addr->sin_family = AF_INET;
	addr->sin_port = htons(TTCPPORT);
	attr.cpu.max = 10;
	attr.mem.npages = 4;
	udpinchan = an_inchan_create(flow, 0, &attr.proto, &attr.addr,
				     &attr.mem, &attr.cpu, ee_udp, 0);
	assert(udpinchan != 0);
	printf("%p: inchannel for TTCP traffic\n", udpinchan);

	ttcpdemux = an_demuxkey_create(an_flow_current());
	assert(ttcpdemux != 0);
	err = an_demuxkey_match(ttcpdemux, ttcppattern.off, ttcppattern.len,
			       ttcppattern.data, 0);
	assert(err == 0);

	strcpy(attr.proto.data, "socket/udp/ip/eth");
	attr.proto.len = strlen(attr.proto.data);
	addr = (struct sockaddr_in *)&attr.addr.socket;
	addr->sin_addr.s_addr = htonl(INADDR_ANY);
	addr->sin_family = AF_INET;
	addr->sin_port = htons(TTCPPORT);
	attr.cpu.max = 10;
	attr.mem.npages = 4;
	ttcpinchan = an_inchan_create(flow, ttcpdemux, &attr.proto,
				      &attr.addr, &attr.mem, &attr.cpu,
				      ee_ttcp, 0);
	assert(ttcpinchan != 0);
	printf("%p: inchannel for TTCP traffic (default pattern)\n",
	       ttcpinchan);
#endif
}

#include <nodeos/synch.h>

#ifdef OSKIT
#include <oskit/machine/proc_reg.h>
#else
#define get_tsc() (0)
#endif
unsigned long long sstamp, estamp;
unsigned tsc_freq = 200;

#ifdef BLOW
static void
ee_sendudp(void)
{
	an_packet_t packet;
	char datums[512];
	an_error_t err;

	memset(datums, 0x69, sizeof datums);
	packet = an_packet_create(an_flow_current(), datums, sizeof datums);
	sstamp = get_tsc();
	while (chandead == 0) {
		err = an_chan_send(udpoutchan, packet);
		assert(err == 0);
		npackets++;
		nbytes += sizeof datums;
		estamp = get_tsc();
	}
}

#else

static void
ee_udp(an_packet_t packet, void *arg)
{
	int psize = an_packet_size(packet);

	if (chandead)
		return;

	if (npackets == 0)
		sstamp = get_tsc();

	npackets++;
	nbytes += psize;
	an_packet_return(packet);

	estamp = get_tsc();
}

static void
ee_ttcp(an_packet_t packet, void *arg)
{
	int psize = an_packet_size(packet);

	if (chandead)
		return;

	if (npackets == 0)
		sstamp = get_tsc();

	npackets++;
	ttcppackets++;
	nbytes += psize;
	an_packet_return(packet);

	estamp = get_tsc();
}
#endif

static void
ee_die(an_packet_t packet, void *arg)
{
	chandead = 1;

#ifdef GPROF
	oskit_stop_gprof();
#endif
	if (sstamp == estamp)
		estamp += tsc_freq;
	printf("Done: %lu bytes, %d packets in %u usec (%u bytes/sec)\n",
	       nbytes, npackets, (unsigned)((estamp - sstamp)/tsc_freq),
	       (unsigned)(((unsigned long long)nbytes*1000000) /
			  ((estamp - sstamp)/tsc_freq)));
	if (ttcppackets)
		printf("      %d default TTCP packets\n", ttcppackets);

	an_packet_return(packet);
	an_flow_destroy(an_flow_current());
}

static void
ee_init(void *arg)
{
	int status;
	int argc = 1;
	char *argv[] = { "UDPtestEE" }; 

	status = ee_main(argc, argv);
}

