/*
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

/*
 * Misc. NodeOS API calls
 */
#ifndef _NODEOS_MISC_H_
#define _NODEOS_MISC_H_

#include <nodeos/impl.h>
#include <nodeos/channel.h>

#define MAX_IFNAME_LEN	8

/*
 * Network interface descriptor.
 * One per hardware interface.
 *
 * ``type'' is the interface type; e.g. ethernet, ATM,
 * ``id'' is a NodeOS assigned unique logical interface number,
 * ``pname'' is the implementation-specific string identifying the interface,
 * ``addr'' identifies this node on the particular interface.
 *
 * The format of address information is interface-specific but will always
 * include a unique address for the node.
 *
 * When creating a channel with proto-spec "if<id>" (e.g., "if0"), the
 * returned address information can be used directly to receive all packets
 * destined for this node on the given interface.  Such packets will have
 * their interface-specific headers stripped before delivery to the channel
 * receive function.  Hence an EE can receive raw packet payloads without
 * needing to know the underlying address format for the hardware.
 * 
 * When creating a channel with proto-spec given by the interface pname
 * (e.g., "eth0"), the channel by default will deliver all packets destined
 * for the node complete with all headers intact.
 */
typedef enum { ANIF_ANY, ANIF_ETH, ANIF_ATM } an_iftype_t;

typedef struct _ifinfo {
	an_iftype_t type;		/* interface type */
	int id;				/* logical interface ID */
	char pname[NODEOS_MAXIFNAME];	/* "physical" name; e.g. eth0, atm1 */
	an_chanaddr_t addr;		/* address info */
} an_ifinfo_t;

/*
 * Retrieve information about network interfaces.
 *
 * ``info'' is an array of an_ifinfo structs to be filled in
 * ``count'' the size of the array on call, the number filled in on return
 *
 * Returns 0 on success, an error otherwise.
 */
an_error_t an_getifinfo(an_ifinfo_t info[], int *count);

/*
 * Get the current time, expressed in seconds and nanoseconds from ??
 */
void an_gettime(an_timespec_t *curtime);

/* XXX compat */
#ifdef NODEOS_SPEC_COMPAT
#define an_timeCurrent an_gettime
#endif

#endif /* _NODEOS_MISC_H_ */
