/*
 * Copyright (c) 1999 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

/*
 * NodeOS Packet API
 *
 * Packets are abstract data buffers.  A variety of operations are provided
 * to access the (possibly discontiguous) memory forming the buffer.
 */
#ifndef _NODEOS_PACKET_H_
#define _NODEOS_PACKET_H_

#include <sys/types.h>
#include <nodeos/flow.h>

typedef struct _an_packet {
	int stuff;
} *an_packet_t;

/* XXX compat */
#ifdef NODEOS_SPEC_COMPAT
#define an_Packet an_packet_t
#define an_packetCreate an_packet_create
#define an_packetDestroy an_packet_destroy
#define an_packetReturn an_packet_return
#define an_packetSize an_packet_size
#define an_packetData an_packet_data
#endif

/*
 * Create a packet associated with the specified flow.
 * The packet's contents are initialized with the given data.
 * Returns the new packet or zero on an error.
 */
an_packet_t an_packet_create(an_flow_t flow, void *buf, size_t len);

/*
 * Destroy a packet, freeing memory.
 * The memory resources are given back to the system NOT returned to
 * any associated in-channel buffer pool.
 */
void an_packet_destroy(an_packet_t packet);

/*
 * Destroy a packet, returning memory to a channel.
 * The packet's memory is put back in the associated in-channel's bufpool.
 * If the packet was not associated with an in-channel, this call is
 * identical to an_packet_destroy.
 */
void an_packet_return(an_packet_t packet);

/*
 * Return the amount of data described by the packet.
 */
int an_packet_size(an_packet_t packet);

/*
 * Return a pointer to the packet's single, contiguous data chunk.
 */
void *an_packet_data(an_packet_t packet);

#endif /* _NODEOS_PACKET_H_ */
