/*
 * Copyright (c) 1999 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

/*
 * The Packet API
 */
#include <errno.h>
#include <nodeos/demuxkey.h>

#include "nodeos.h"
#include "flow.h"
#include "demux_tlist.h"
#include "mem.h"

#define MAX_MATCHES	7 /* XXX */

an_demuxkey_t
an_demuxkey_create(an_flow_t flow)
{
	demux_tlist_t *tlist;
	int err;

	assert((flow_t *)flow == flow_current());

	NODEOS_ENTER;
	err = demux_tlist_create(MAX_MATCHES, &tlist);
	NODEOS_LEAVE;
	return err ? 0 : (an_demuxkey_t)tlist;
}

void
an_demuxkey_destroy(an_demuxkey_t key)
{
	NODEOS_ENTER;
	demux_tlist_destroy((demux_tlist_t *)key);
	NODEOS_LEAVE;
}

an_error_t
an_demuxkey_match(an_demuxkey_t key, int off, int len, void *datap, void *maskp)
{
	void *data = 0;
#if 0
	void *mask = 0;
#endif
	int err = 0;

	NODEOS_ENTER;

	if (datap) {
		data = mem_alloc(len);
		if (data) {
			NODEOS_COPYIN(data, datap, len, &err);
		} else
			err = ENOMEM;
	} else
		err = EINVAL;
	if (err)
		goto done;

#if 0
	if (maskp) {
		mask = mem_alloc(len);
		if (mask) {
			NODEOS_COPYIN(mask, maskp, len, &err);
		} else
			err = ENOMEM;
	} else
		mask = 0;
	if (err)
		goto done;
#endif

	err = demux_tlist_append((demux_tlist_t *)key, off, len, data);

done:
	NODEOS_LEAVE;
	return (an_error_t)err;
}
