/*
 * Copyright (c) 1999 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

/*
 * The Flow API
 */
#include <nodeos/impl.h>
#include <nodeos/flow.h>

#include "nodeos.h"
#include "flow.h"
#include "credential.h"
#include "resource.h"

an_flow_t
an_flow_create(an_cred_t cred, an_cpuspec_t *cpu, an_memspec_t *mem,
	       an_flowinit_t init, an_flowterm_func_t term)
{
	flow_t *newflow;
	int err;

	NODEOS_ENTER;

	/*
	 * Create a flow object associated with the given credential and
	 * requiring the indicated resources.
	 */
	err = flow_create((struct credential *)cred, cpu, mem,
			  init, term, &newflow);

	NODEOS_LEAVE;
	return err ? 0 : (an_flow_t)newflow;
}

void
an_flow_destroy(an_flow_t flow)
{
	int err;

	NODEOS_ENTER;
	err = flow_destroy((flow_t *)flow);
	assert(err == 0);
	NODEOS_LEAVE;
}

an_flow_t
an_flow_current(void)
{
	an_flow_t id;

	NODEOS_ENTER;
	id = (an_flow_t)flow_current();
	NODEOS_LEAVE;
	return id;
}

