/*
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

/*
 * The Mempool API
 */
#include <errno.h>
#include <nodeos/impl.h>
#include <nodeos/mempool.h>

#include "nodeos.h"
#include "mempool.h"

an_mempool_t
an_mempool_create(an_mempool_t parent, an_mempoolfull_func_t func)
{
	mempool_t *newpool, *iparent;
	int err;

	NODEOS_ENTER;

	newpool = 0;
	if (parent) {
		iparent = NODEOS_INTERN_MEMPOOL(parent);
		if (iparent == 0) {
			err = EINVAL;
			goto done;
		}
	}

	newpool = mempool_alloc(mempool_current(), sizeof *newpool);
	if (newpool == 0) {
		err = ENOMEM;
		goto done;
	}

	err = mempool_create(newpool, iparent, func);

 done:
	if (err && newpool)
		mempool_free(mempool_current(), newpool, sizeof *newpool);
	NODEOS_LEAVE;
	return err ? 0 : NODEOS_EXTERN_MEMPOOL(newpool);
}

an_error_t
an_mempool_destroy(an_mempool_t mpool)
{
	mempool_t *impool;
	int err = 0;

	NODEOS_ENTER;
	impool = NODEOS_INTERN_MEMPOOL(mpool);
	if (impool == 0) {
		err = EINVAL;
		goto done;
	}
	err = mempool_destroy(impool);
	if (err == 0)
		mempool_free(mempool_current(), impool, sizeof *impool);
 done:
	NODEOS_LEAVE;
	return (an_error_t)err;
}

an_mempool_t
an_mempool_current(void)
{
	an_mempool_t id;

	NODEOS_ENTER;
	id = NODEOS_EXTERN_MEMPOOL(mempool_current());
	NODEOS_LEAVE;
	return id;
}

void *
an_mempool_alloc(an_mempool_t mpool, size_t size)
{
	mempool_t *impool;
	void *ptr = 0;

	NODEOS_ENTER;
	impool = NODEOS_INTERN_MEMPOOL(mpool);
	if (impool != 0)
		ptr = mempool_alloc(impool, size);
	NODEOS_LEAVE;
	return ptr;
}

void
an_mempool_free(an_mempool_t mpool, void *ptr, size_t size)
{
	mempool_t *impool;

	NODEOS_ENTER;
	impool = NODEOS_INTERN_MEMPOOL(mpool);
	if (impool != 0)
		mempool_free(impool, ptr, size);
	NODEOS_LEAVE;
}
