/*
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

/*
 * Misc. API functions
 */
#include <errno.h>

#include <nodeos/misc.h>
#include "misc.h"
#include "mem.h"

#include "nodeos.h"

void
an_gettime(an_timespec_t *curtime)
{
	NODEOS_ENTER;
	time_current(curtime);
	NODEOS_LEAVE;
}

an_error_t
an_getifinfo(an_ifinfo_t info[], int *count)
{
	int err;
	int icount, num;
	an_ifinfo_t *iinfo;

	NODEOS_ENTER;

	/*
	 * Validate arguments.
	 */
	if (count == 0) {
		err = EINVAL;
		goto done; 
	}

	/*
	 * Read desired size.
	 */
	NODEOS_COPYIN(&icount, count, sizeof icount, &err);
	if (err)
		goto done;

	/*
	 * Get a pointer to the info
	 */
	num = icount;
	iinfo = if_getinfo(&num);

	/*
	 * Copy out data
	 */
	NODEOS_COPYOUT(count, &num, sizeof *count, &err);
	if (err == 0 && iinfo != 0)
		NODEOS_COPYOUT(info, iinfo, sizeof *iinfo * num, &err);

done:
	NODEOS_LEAVE;
	return (an_error_t)err;
}

