/*
 * Copyright (c) 1999 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

/*
 * The Packet API
 */
#include <nodeos/packet.h>

#include "nodeos.h"
#include "flow.h"
#include "packet.h"

an_packet_t
an_packet_create(an_flow_t flow, void *buf, size_t len)
{
	packet_t *packet;
	int err;

	NODEOS_ENTER;
	err = packet_create((flow_t *)flow, 0, buf, len, &packet);
	NODEOS_LEAVE;
	return err ? 0 : (an_packet_t)packet;
}

void
an_packet_destroy(an_packet_t packet)
{
	NODEOS_ENTER;
	packet_destroy((packet_t *)packet);
	NODEOS_LEAVE;
}

void
an_packet_return(an_packet_t packet)
{
	NODEOS_ENTER;
	packet_return((packet_t *)packet);
	NODEOS_LEAVE;
}

int
an_packet_size(an_packet_t packet)
{
	int size;

	NODEOS_ENTER;
	size = packet_size((packet_t *)packet);
	NODEOS_LEAVE;

	return size;
}

void *
an_packet_data(an_packet_t packet)
{
	void *data;

	NODEOS_ENTER;
	data = packet_data((packet_t *)packet);
	NODEOS_LEAVE;

	return data;
}
