/*
 * Copyright (c) 1999 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

#include <nodeos/synch.h>

#include "nodeos.h"
#include "synch.h"

an_mutex_t
an_mutex_create(void)
{
	mutex_t *mutex;
	int err;

	NODEOS_ENTER;
	err = mutex_create(&mutex);
	NODEOS_LEAVE;
	return err ? 0 : (an_mutex_t)mutex;
}

void
an_mutex_lock(an_mutex_t mutex)
{
	NODEOS_ENTER;
	mutex_lock((mutex_t *)mutex);
	NODEOS_LEAVE;
}

an_error_t
an_mutex_trylock(an_mutex_t mutex)
{
	int err;

	NODEOS_ENTER;
	err = mutex_trylock((mutex_t *)mutex);
	NODEOS_LEAVE;
	return (an_error_t)err;
}

void
an_mutex_unlock(an_mutex_t mutex)
{
	NODEOS_ENTER;
	mutex_unlock((mutex_t *)mutex);
	NODEOS_LEAVE;
}

an_error_t
an_mutex_destroy(an_mutex_t mutex)
{
	int err;

	NODEOS_ENTER;
	err = mutex_destroy((mutex_t *)mutex);
	NODEOS_LEAVE;
	return (an_error_t)err;
}


an_cond_t
an_cond_create(void)
{
	cond_t *cond;
	int err;

	NODEOS_ENTER;
	err = cond_create(&cond);
	NODEOS_LEAVE;
	return err ? 0 : (an_cond_t)cond;
}

void an_cond_wait(an_cond_t cond, an_mutex_t mutex)
{
	NODEOS_ENTER;
	cond_wait((cond_t *)cond, (mutex_t *)mutex);
	NODEOS_LEAVE;
}

void
an_cond_signal(an_cond_t cond)
{
	NODEOS_ENTER;
	cond_signal((cond_t *)cond);
	NODEOS_LEAVE;
}

void
an_cond_broadcast(an_cond_t cond)
{
	NODEOS_ENTER;
	cond_broadcast((cond_t *)cond);
	NODEOS_LEAVE;
}

an_error_t
an_cond_destroy(an_cond_t cond)
{
	int err;
	NODEOS_ENTER;
	err = cond_destroy((cond_t *)cond);
	NODEOS_LEAVE;
	return (an_error_t)err;
}
