/*
 * Copyright (c) 1999 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */
#include <errno.h>
#include <string.h>
#include <assert.h>
#include <sys/types.h>

#include "demux.h"
#include "packet.h"
#include "mem.h"

void
demux_init(void)
{
}

int
demux_create(demux_id_t id, demux_func_t func, void *data, demux_t **out_demux)
{
	demux_t *demux;

	demux = mem_alloc(sizeof *demux);
	if (demux == 0)
		return ENOMEM;

	demux->id = id;
	demux->func = func;
	demux->data = data;
	*out_demux = demux;

	return 0;
}

void
demux_destroy(demux_t *demux)
{
	mem_free(demux, sizeof *demux);
}

/*
 * Add a demux descriptor to a list.
 * Should probably return an error for "conflicting" demuxes
 * (if there is such a thing).
 *
 * The list is assumed to be locked by the caller.
 */
int
demux_add(demux_t **list, demux_t *demux, int prio)
{
	demux_t *dm, **dmp;

	for (dmp = list; (dm = *dmp) != 0; dmp = &dm->next)
		if (prio > dm->prio)
			break;

	demux->next = dm;
	*dmp = demux;
	return 0;
}

/*
 * Remove a demux descriptor from a list.
 * The list is assumed to be locked by the caller.
 */
void
demux_remove(demux_t **list, demux_t *demux)
{
	demux_t **dmp;

	for (dmp = list; *dmp != 0; dmp = &(*dmp)->next)
		if (*dmp == demux) {
			*dmp = demux->next;
			break;
		}
}

demux_t *
demux_find(demux_t *list, demux_id_t id)
{
	demux_t *dm;

	for (dm = list; dm != 0; dm = dm->next)
		if (dm->id == id)
			return dm;
	return 0;
}


/*
 * Apply demux routines from the list until one matches.
 * Returns the id value of the first matching demux or 0 otherwise.
 * The list is assumed to be locked by the caller.
 */
demux_id_t
demux_apply(demux_t *list, void *data, int len)
{
	demux_t *dm;

	for (dm = list; dm != 0; dm = dm->next)
		if (dm->func == 0 || dm->func(dm->data, data, len) != 0)
			return dm->id;
	return 0;
}
