/*
 * Copyright (c) 1999 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

/*
 * Internal locking, just use pthreads sans errors
 */
#ifndef _ILOCK_IMPL_H_
#define _ILOCK_IMPL_H_

#include <assert.h>
#include <pthread.h>

typedef pthread_mutex_t ilock_t;
typedef pthread_cond_t icond_t;

#define ilock_init(l) { \
	int err = pthread_mutex_init((l), 0); \
	assert(err == 0); \
}

#define ilock_deinit(l) { \
	int err = pthread_mutex_destroy((l)); \
	assert(err == 0); \
}

#define ilock_lock(l) { \
	int err = pthread_mutex_lock((l)); \
	assert(err == 0); \
}

#define ilock_trylock(l)  \
	(pthread_mutex_trylock((l)))

#define ilock_unlock(l) { \
	int err = pthread_mutex_unlock((l)); \
	assert(err == 0); \
}

#define ilock_islocked(l) \
	(pthread_mutex_trylock((l)) == 0 ? (pthread_mutex_unlock((l)), 0) : 1)

#define icond_init(c) { \
	int err = pthread_cond_init((c), 0); \
	assert(err == 0); \
}

#define icond_deinit(c) { \
	int err = pthread_cond_destroy((c)); \
	assert(err == 0); \
}

#define icond_wait(c, l) { \
	int err; \
 	pthread_cleanup_push((void (*)(void *))pthread_mutex_unlock, (l)); \
	err = pthread_cond_wait((c), (l)); \
	pthread_cleanup_pop(0); \
	assert(err == 0); \
}

#define icond_signal(c) { \
	int err = pthread_cond_signal((c)); \
	assert(err == 0); \
}

#define icond_broadcast(c) { \
	int err = pthread_cond_broadcast((c)); \
	assert(err == 0); \
}

#endif /* _ILOCK_IMPL_H_ */
