/*
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <assert.h>

#include "resource.h"
#include "credential.h"
#include "thread.h"
#include "flow.h"
#include "mem.h"
#include "mempool.h"
#include "channel.h"
#include "packet.h"
#include "ilock.h"
#include "os.h"

void load_ee(void *);
void nodeos_die(void *);

#include "version.h"
int version = VERSION;

static icond_t cond;
static ilock_t lock;
static int childdead;
int bootstrapped;

int
main()
{
	os_init_basic();
	os_init_mem();
	printf("Utah NodeOS (version 0.%04d)\n", version);
	os_init_devices();

	/*
	 * Initialize NodeOS components.
	 */
	mem_init();
	thread_init();
	credential_init();
	mempool_init();
	channel_init();
	packet_init();

	ilock_init(&lock);
	icond_init(&cond);
	ilock_lock(&lock);

	/*
	 * Create our initial flow.
	 * This will fire off a thread at load_ee which takes over from here.
	 * All memory from this point on is charged to some flow.
	 */
	flow_init(load_ee, nodeos_die);

	/*
	 * Wait til the child flow dies
	 */
	while (childdead == 0)
		icond_wait(&cond, &lock);
	bootstrapped = 0;
	printf("Initial EE died, exiting...\n");

	/*
	 * XXX wait a while longer to allow things to shutdown cleanly.
	 */
	{
		an_timespec_t ts;

		ts.tv_sec = 1;
		ts.tv_nsec = 0;
		thread_sleep(&ts);
	}

	exit(0);
}

void
load_ee(void *arg)
{
	void (*func)(void *);
	credential_t *cred;
	cpuspec_t cspec;
	memspec_t mspec;
	flow_t *flow;
	int err;

	bootstrapped = 1;
	printf("Loading initial EE...\n");
#if 1
	{
		extern credential_t *user_cred;
		extern an_flowinit_t InitEE;

		func = InitEE;
		cred = user_cred;
		memset(&cspec, 0, sizeof cspec);
		cspec.max = 20;
		cspec.stacksize = 32 * 1024;
		mspec.npages = 1000;
		mspec.pool = (an_mempool_t)mempool_current();
	}
#endif
	err = flow_create(cred, &cspec, &mspec, func, nodeos_die, &flow);
	assert(err == 0);

	thread_exit();
}

void
nodeos_die(void *arg)
{
	ilock_lock(&lock);
	childdead = 1;
	icond_signal(&cond);
	ilock_unlock(&lock);
}
