/*
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

#ifndef _MEMPOOL_H_
#define _MEMPOOL_H_

#include <nodeos/impl.h>

#include "hnode.h"

/* XXX */
#define mempool_cbfunc_t	an_mempoolfull_func_t

struct flow;
struct resource;

/*
 * The mempool: aggregates the memory resources from multiple flows.
 */
struct mempool {
	struct hnode me;	/* must be first */
	struct hnode flowlist;	/* all flows associated with this pool */
	int physcurrent;	/* physpages in use by flows in this pool */
	int physlimit;		/* physpages available to flows in this pool */
	mempool_cbfunc_t callback; /* reclaimation callback */
};
typedef struct mempool mempool_t;

void mempool_init(void);

/*
 * Get the mempool associated with the current thread's flow.
 */
mempool_t *mempool_current(void);

/*
 * Create a new mempool with no associated flows.  The parent
 * parameter defines this pool's position in the pool hierarchy.
 * The callback routine is invoked whenever the NodeOS detects that
 * the indicated pool is over its page limit.  Parameters to the
 * callback function include the pool and some indication of how
 * much memory needs to be returned.  The callback function should
 * use unmapregion to free up memory.
 */
int mempool_create(mempool_t *mpool, mempool_t *parent,
		   mempool_cbfunc_t callback);

/*
 * Destroy a mempool.  All flows that were in the pool are implicitly
 * removed (see an_mem_removeflow) and all child pools are (recursively)
 * destroyed.
 */
int mempool_destroy(mempool_t *pool);

/*
 * Donates the memory resources associated with a flow to the pool.
 */
void mempool_addflow(mempool_t *pool, struct flow *flow);

/*
 * Removes the memory resources associated with a flow from the pool.
 */
void mempool_removeflow(mempool_t *pool, struct flow *flow);

/*
 * Simple malloc/free allocation interface
 */
void *mempool_alloc(mempool_t *pool, size_t size);
void mempool_free(mempool_t *pool, void *ptr, size_t size);

#endif /* _MEMPOOL_H_ */
