/*
 * Copyright (c) 1999 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

#ifndef _NODEOS_NODEOS_IMPL_H_
#define _NODEOS_NODEOS_IMPL_H_

#include <pthread.h>
#include <string.h>
#include <signal.h>
#include <assert.h>

#include "thread.h"
extern sigset_t thread_intrmask;

/*
 * We disallow both thread interrupts (signals) and thread dead (cancelation)
 * while in the NodeOS.  Both are checked on exit from the NodeOS.
 */
#define NODEOS_ENTER { \
	thread_t *th = thread_current(); \
	int ostate; \
	pthread_sigmask(SIG_BLOCK, &thread_intrmask, 0); \
	pthread_setcanceltype(PTHREAD_CANCEL_DEFERRED, &ostate); \
	assert(ostate == PTHREAD_CANCEL_ASYNCHRONOUS); \
	assert((th->flags & THREAD_INKERNEL) == 0); \
	th->flags |= THREAD_INKERNEL; \
}

#define NODEOS_LEAVE { \
	thread_t *th = thread_current(); \
	int ostate; \
	pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS, &ostate); \
	assert(ostate == PTHREAD_CANCEL_DEFERRED); \
	pthread_testcancel(); \
	assert((th->flags & THREAD_INKERNEL) != 0); \
	th->flags &= ~THREAD_INKERNEL; \
	if (th->flags & THREAD_INTSON) \
		pthread_sigmask(SIG_UNBLOCK, &thread_intrmask, 0); \
}

#define NODEOS_COPYIN(ndst, usrc, len, errp) { \
	memcpy((ndst), (usrc), (len)); \
	*errp = 0; \
}

#define NODEOS_COPYOUT(udst, nsrc, len, errp) { \
	memcpy((udst), (nsrc), (len)); \
	*errp = 0; \
}

/*
 * Convert from the external representation (e.g., an object descriptor)
 * to an internal representation (e.g., a struct pointer) or visa-versa.
 *
 * In our implementation, we just pass pointers.
 */
#define NODEOS_INTERNALIZE(e_ob, cast)	((cast)(e_ob))

#define NODEOS_INTERN_CHAN(o)		NODEOS_INTERNALIZE((o), channel_t *)
#define NODEOS_INTERN_CRED(o)		NODEOS_INTERNALIZE((o), credential_t *)
#define NODEOS_INTERN_DEMUXKEY(o)	NODEOS_INTERNALIZE((o), demux_tlist_t *)
#define NODEOS_INTERN_FLOW(o)		NODEOS_INTERNALIZE((o), flow_t *)
#define NODEOS_INTERN_MEMPOOL(o)	NODEOS_INTERNALIZE((o), mempool_t *)
#define NODEOS_INTERN_PACKET(o)		NODEOS_INTERNALIZE((o), packet_t *)
#define NODEOS_INTERN_MUTEX(o)		NODEOS_INTERNALIZE((o), mutex_t *)
#define NODEOS_INTERN_COND(o)		NODEOS_INTERNALIZE((o), cond_t *)
#define NODEOS_INTERN_THREAD(o)		NODEOS_INTERNALIZE((o), thread_t *)

#define NODEOS_EXTERNALIZE(i_o, cast)	((cast)(i_o))

#define NODEOS_EXTERN_CHAN(o)		NODEOS_EXTERNALIZE((o), an_chan_t)
#define NODEOS_EXTERN_CRED(o)		NODEOS_EXTERNALIZE((o), an_cred_t)
#define NODEOS_EXTERN_DEMUXKEY(o)	NODEOS_EXTERNALIZE((o), an_demuxkey_t)
#define NODEOS_EXTERN_FLOW(o)		NODEOS_EXTERNALIZE((o), an_flow_t)
#define NODEOS_EXTERN_MEMPOOL(o)	NODEOS_EXTERNALIZE((o), an_mempool_t)
#define NODEOS_EXTERN_PACKET(o)		NODEOS_EXTERNALIZE((o), an_packet_t)
#define NODEOS_EXTERN_MUTEX(o)		NODEOS_EXTERNALIZE((o), an_mutex_t)
#define NODEOS_EXTERN_COND(o)		NODEOS_EXTERNALIZE((o), an_cond_t)
#define NODEOS_EXTERN_THREAD(o)		NODEOS_EXTERNALIZE((o), an_thread_t)

#endif /* _NODEOS_NODEOS_IMPL_H_ */
