/*
 * Copyright (c) 1999 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */
#include <errno.h>
#include <string.h>
#include <assert.h>
#include <sys/types.h>

#include "flow.h"
#include "channel.h"
#include "packet.h"
#include "mem.h"

void
packet_init(void)
{
}

int
packet_create(flow_t *flow, channel_t *chan, void *buf, size_t len,
	      packet_t **out_packet)
{
	packet_t *packet;

	packet = mem_alloc(sizeof *packet + len);
	if (packet == 0)
		return ENOMEM;

	packet->chan = chan;
	packet->ffunc = 0;
	packet->size = len;
	packet->bsize = len;
	memcpy(packet->data, buf, len);

	*out_packet = packet;
	return 0;
}

int
packet_create_external(flow_t *flow, channel_t *chan, void *buf, size_t len,
		       off_t offset, packet_free_t ffunc, size_t blen,
		       packet_t **out_packet)
{
	packet_t *packet;

	assert(ffunc != 0);

	packet = mem_alloc(sizeof *packet + sizeof(void *));
	if (packet == 0)
		return ENOMEM;

	packet->chan = chan;
	packet->ffunc = ffunc;
	packet->bsize = blen;
	packet->size = len;
	packet->offset = offset;
	*(void **)packet->data = buf;

	*out_packet = packet;
	return 0;
}

void
packet_destroy(packet_t *packet)
{
	if (packet->ffunc) {
		packet->ffunc(packet->chan,
			      *(void **)packet->data, packet->bsize);
		mem_free(packet, sizeof *packet + sizeof(void *));
	} else
		mem_free(packet, sizeof *packet + packet->size);
}

void
packet_return(packet_t *packet)
{
	if (packet->ffunc) {
		struct chanbuf *cbuf;

		assert(packet->bsize == sizeof *cbuf);
		cbuf = *(struct chanbuf **)packet->data;
		channel_addbuf(packet->chan, cbuf);
		mem_free(packet, sizeof *packet + sizeof(void *));
	} else
		mem_free(packet, sizeof *packet + packet->size);
}
