/*
 * Copyright (c) 1999 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

#ifndef _PACKET_IMPL_H_
#define _PACKET_IMPL_H_

struct channel;
struct flow;

typedef void (*packet_free_t)(struct channel *chan, void *ptr, size_t size);

struct packet {
	packet_free_t ffunc;	/* external free function or 0 */
	struct channel *chan;	/* for memory accounting */
	size_t bsize;		/* buffer size for external bufs */
	size_t size;		/* size of following data */
	off_t offset;		/* offset into the data buffer */
	char data[0];
};
typedef struct packet packet_t;

void packet_init(void);
int packet_create(struct flow *flow, struct channel *chan,
		  void *buf, size_t len, packet_t **out_packet);
int packet_create_external(struct flow *flow, struct channel *chan,
			   void *buf, size_t len, off_t offset,
			   packet_free_t ffunc, size_t blen,
			   packet_t **out_packet);
void packet_destroy(packet_t *packet);
void packet_return(packet_t *packet);

#define packet_size(pack) ((pack)->size)
#define packet_data(pack) \
	((pack)->ffunc ? (*(char **)(pack)->data+(pack)->offset) : (pack)->data)
#define packet_bufsize(pack) ((pack)->bsize)

#endif /* _PACKET_IMPL_H_ */
