/*
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

#ifndef _THREAD_IMPL_H_
#define _THREAD_IMPL_H_

#include <pthread.h>
#include <sched.h>
#include <setjmp.h>
#include <nodeos/impl.h>

#include "ilock.h"
#include "synch.h"
#include "resource.h"

/*
 * Stuff That Doesn't Belong
 * XXX hack around pthread deficiencies:
 *	OSKit doesn't support attribute scope stuff currently,
 *		the default (only!) scope is system.
 *	FreeBSD doesn't support cancelation,
 *		the EE had really better cooperate!
 */
#ifndef PTHREAD_CANCEL_ASYNCHRONOUS
#define PTHREAD_CANCEL_ASYNCHRONOUS	0
#define PTHREAD_CANCEL_DEFERRED		0
#define pthread_setcanceltype(t, sp)	(*(sp) = 0)
#define pthread_cancel(t)		0
#define pthread_testcancel()
#undef PTHREAD_SCOPE_SYSTEM /* XXX freebsd defines this but doesn't impl */
#endif

#ifndef PTHREAD_SCOPE_SYSTEM
#define pthread_attr_setscope(a,s) (0)
#endif

/* XXX */
#define thread_handler_t	an_thread_intr_t

struct threadpool;
struct channel;

typedef void (*thread_func_t)(void *, void *);

struct threadwork {
	struct threadwork *next;/* next item on list */
	int inkern;		/* function should run in kernel context */
	struct channel *chan;	/* channel to associate thread with */
	thread_func_t func;	/* function the thread should start in */
	void *farg[2];		/* and arguments for that function */
};

struct thread {
	struct thread *next;	 /* threadpool link */
	struct thread *freenext; /* threadpool freelist link */
	struct threadpool *pool; /* our threadpool */
	struct channel *chan;	 /* channel we are processing for */
	pthread_t id;		 /* pthread ID */
	struct sched_param pprio;/* pthread scheduling priority */
	int nprio;		 /* NodeOS scheduling priority */
	icond_t waiting;	 /* place for this thread to wait */
	icond_t waitedon;	 /* place for others to wait for this thread */
	struct threadwork *work; /* work targetted to this thread */
	thread_handler_t handler;/* interrupt/trap handler */
	ilock_t lock;		 /* lock to protect interrupt state */
	cond_t *cond;		 /* non-zero if in cond_wait */
	int flags;		 /* various flags */
	sigjmp_buf jmpbuf;	 /* warp the thread back to its origin */
};
typedef struct thread thread_t;

#define THREAD_INKERNEL	0x01	/* thread is in the NodeOS */
#define THREAD_WAITEDON	0x02	/* someone is waiting for thread */
#define THREAD_DONE	0x04	/* waiter is finished with the thread */
#define THREAD_INTSON	0x08	/* interrupts should be allowed */

#define thread_lock(t)		ilock_lock(&(t)->lock)
#define thread_unlock(t)	ilock_unlock(&(t)->lock)

void thread_init(void);
thread_t *thread_current(void);
int thread_checkattrs(cpuspec_t *cspec);
int thread_spawn(struct threadpool *tpool, thread_t **out_thread);
void thread_kill(thread_t *thread);
void thread_exit(void);
void thread_yield(void);
void thread_sleep(an_timespec_t *delay);
void *thread_getdata(void);
void thread_setdata(void *);
void thread_interrupt(thread_t *thread);
void thread_block_interrupt(void);
void thread_unblock_interrupt(void);
void thread_sethandler(thread_handler_t handler);
void thread_condwait(cond_t *cond);
int thread_getprio(thread_t *thread);
int thread_setprio(thread_t *thread, int prio);

#endif /* _THREAD_IMPL_H_ */

