/*
 * Copyright (c) 1999 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */
#include "options.h"

#ifdef CHAN_ETH

#include <stdio.h>
#include <errno.h>
#include <stdlib.h>
#include <string.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <net/if.h>
#include <netinet/in.h>
#include <netinet/if_ether.h>

#ifdef __FreeBSD__
#include <net/if_dl.h>
#endif

#include "unix_eth.h"

#define max(x, y) ((x) > (y) ? (x) : (y))

int
unix_eth_get_if_info(struct if_info **nfo)
{
	int fd, len, lastlen, n, flags;
	char *buf, *ptr, *cptr;
	struct ifconf ifc;
	struct ifreq *ifr, ifrcopy;
	char lastname[IFNAMSIZ];
	struct if_info *ifnfo;

	if ( !nfo || !(fd = socket(PF_INET, SOCK_DGRAM, 0)))
		return 0;

	lastlen = 0;
	len = 10 * sizeof(struct ifreq);

	while (1) {
		if ( !(buf = (char *)malloc(len)))
			return 0;
		
		ifc.ifc_len = len;
		ifc.ifc_buf = buf;
		
		if (ioctl(fd, SIOCGIFCONF, &ifc) < 0) {
			if (errno != EINVAL || lastlen != 0) {
				free(buf);
				return 0;
			}
		}	
		
		if (ifc.ifc_len == lastlen)
			break;
		
		lastlen = ifc.ifc_len;
		len += 10 * sizeof(struct ifreq);
			
		free(buf);
	}

	lastname[0] = 0;

	ifnfo = malloc((ifc.ifc_len
		      / (sizeof(struct sockaddr) + sizeof(ifr->ifr_name)))
		      * sizeof(struct if_info));
	if (!ifnfo)
		return 0;
	
	for (n = 0, ptr = buf; ptr < buf + ifc.ifc_len; ) {
		ifr = (struct ifreq *)ptr;

#ifdef HAVE_SOCKADDR_SA_LEN		   
		len = max(sizeof(struct sockaddr), ifr->ifr_addr.sa_len);
#else
		len = sizeof(struct sockaddr);
#endif
		ptr += sizeof(ifr->ifr_name) + len;

		/*
		 * Eliminate interface aliases.
		 */
		if ( (cptr = (char *)strchr(ifr->ifr_name, ':')) != NULL)
			*cptr = 0;
		if (strncmp(lastname, ifr->ifr_name, IFNAMSIZ) == 0) 
			continue;
		
		strncpy(lastname, ifr->ifr_name, IFNAMSIZ);

		ifrcopy = *ifr;
		if (ioctl(fd, SIOCGIFFLAGS, &ifrcopy) < 0) {
#if 0
			Panic("ioctl error");
#else
			perror("error getting IFFLAGS");
#endif			      
		}
		
		flags = ifrcopy.ifr_flags;
		
		if ((flags & IFF_UP) == 0 || (flags & IFF_LOOPBACK))
			continue;

		memcpy(ifnfo[n].name, lastname, IFNAMSIZ);

		     
#ifdef __linux__
		if (ioctl(fd, SIOCGIFHWADDR, &ifrcopy) < 0) {
			perror("error getting HWADDR");
		}
		memcpy(ifnfo[n].addr, &ifrcopy.ifr_hwaddr.sa_data[0], 6);
#elif __FreeBSD__
		if (ifr->ifr_addr.sa_family == AF_LINK) {
			struct sockaddr_dl *dla;

			dla = (struct sockaddr_dl *) &ifr->ifr_addr;
			memcpy(ifnfo[n].addr,  LLADDR(dla), dla->sdl_alen);
		} else {
			if (ioctl(fd, SIOCGIFADDR, &ifrcopy) < 0) 
				perror("error getting HWADDR");
			memcpy(ifnfo[n].addr,
			       &ifrcopy.ifr_addr.sa_data, ETHER_ADDR_LEN);
		}
#else		
		if (ioctl(fd, SIOCGIFADDR, &ifrcopy) < 0) {
			perror("ioctl error");
		}

		sin = (struct sockaddr_in *) &arpreq.arp_pa;
		memset(sin, 0, sizeof(struct sockaddr_in));
		sin->sin_family = AF_INET;
		memcpy(&sin->sin_addr, ifrcopy.ifr_addr,
		       sizeof (struct in_addr));

		if (ioctl(fd, SIOCGARP, &arpreq) < 0) {
			perror("ioctl error");
		}

		memcpy(ifnfo[n].addr, arpreq.arp_ha.sa_data, 6);
#endif		      

		n++;
	}

	*nfo = ifnfo;
	
	return n;
}

#endif /* CHAN_ETH */
